//
//  AppSearchViewController.swift
//  App Store Search
//
//  Created by Nikita Somenkov on 28.01.2022.
//

import Foundation
import UIKit

@MainActor
final class AppSearchViewController: UIViewController {

    // MARK: - Neested Types

    enum Section: Hashable {
        case main
    }

    enum Cell: Hashable {
        case app(AppEnity)
    }

    enum State {
        case initial
        case loading
        case empty
        case data([AppEnity])
        case error(Error)
    }

    typealias Snapshot = NSDiffableDataSourceSnapshot<Section, Cell>
    typealias DataSource = UITableViewDiffableDataSource<Section, Cell>

    // MARK: - Subviews

    private lazy var tableView = UITableView(frame: .zero, style: .insetGrouped)
    private lazy var searchController = UISearchController(searchResultsController: nil)

    private lazy var activityIndicatorView = UIActivityIndicatorView()
    private lazy var statusLabel = UILabel()

    // MARK: - Private Properties

    private lazy var dataSource = createDataSource()

    private var searchingTask: Task<Void, Never>?
    private lazy var searchService = AppsSearchService()
    private var state: State = .initial {
        didSet { updateState() }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        configureApperance()
    }

}

private extension AppSearchViewController {

    func configureApperance() {
        configureView()
        configureSubviews()
        configureConstraints()
        configureNavigation()
    }

    func configureNavigation() {
        navigationItem.title = "Search"
        navigationItem.searchController = searchController
    }

    func configureView() {
        view.backgroundColor = .systemGroupedBackground

        view.addSubview(tableView)
        view.addSubview(activityIndicatorView)
        view.addSubview(statusLabel)
    }

    func configureSubviews() {
        tableView.dataSource = dataSource
        tableView.register(AppSearchCell.self, forCellReuseIdentifier: "app")
        tableView.allowsSelection = false
        tableView.contentInset.top = 64

        statusLabel.textColor = .secondaryLabel
        statusLabel.textAlignment = .center
        statusLabel.numberOfLines = 0

        searchController.searchBar.delegate = self
        searchController.searchBar.placeholder = "Search apps"
        searchController.searchBar.autocapitalizationType = .none

        updateState()
    }

    func configureConstraints() {
        tableView.translatesAutoresizingMaskIntoConstraints = false
        activityIndicatorView.translatesAutoresizingMaskIntoConstraints = false
        statusLabel.translatesAutoresizingMaskIntoConstraints = false

        NSLayoutConstraint.activate([
            tableView.topAnchor.constraint(equalTo: view.topAnchor),
            tableView.bottomAnchor.constraint(equalTo: view.bottomAnchor),
            tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor),

            activityIndicatorView.bottomAnchor.constraint(equalTo: statusLabel.topAnchor, constant: -16),
            activityIndicatorView.centerXAnchor.constraint(equalTo: statusLabel.centerXAnchor),

            statusLabel.bottomAnchor.constraint(equalTo: view.centerYAnchor, constant: -56),
            statusLabel.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 16),
            statusLabel.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -16),
        ])
    }

    func createDataSource() -> DataSource {
        DataSource(tableView: tableView, cellProvider: { (tableView, indexPath, item) -> UITableViewCell? in
            switch item {
            case .app(let appEnity):
                let cell = tableView.dequeueReusableCell(withIdentifier: "app", for: indexPath) as? AppSearchCell
                cell?.configure(with: appEnity)
                return cell
            }
        })
    }

    func updateState() {
        switch state {
        case .initial:
            tableView.isHidden = false
            activityIndicatorView.stopAnimating()
            statusLabel.text = "Input your request"
        case .loading:
            tableView.isHidden = true
            activityIndicatorView.startAnimating()
            statusLabel.text = "Loading..."
        case .empty:
            tableView.isHidden = true
            activityIndicatorView.stopAnimating()
            statusLabel.text = "No apps found"
        case .data(let apps):
            tableView.isHidden = false
            activityIndicatorView.stopAnimating()
            statusLabel.text = nil

            var snapshot = Snapshot()

            snapshot.appendSections([.main])
            snapshot.appendItems(apps.map { .app($0) }, toSection: .main)

            dataSource.apply(snapshot)
        case .error(let error):
            tableView.isHidden = true
            activityIndicatorView.stopAnimating()
            statusLabel.text = "Error: \(error.localizedDescription)"
        }
    }

}

extension AppSearchViewController: UISearchControllerDelegate, UISearchBarDelegate {

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        guard let query = searchBar.text else {
            return
        }

        searchingTask?.cancel()
        searchingTask = Task { [weak self] in
            self?.state = .loading

            do {
                let apps = try await searchService.search(with: query)

                if Task.isCancelled {
                    return
                }

                if apps.isEmpty {
                    self?.state = .empty
                } else {
                    self?.state = .data(apps)
                }
            } catch {
                self?.state = .error(error)
            }
        }
    }

}
